/*
 * Copyright (c) 2003 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "wmclient.h"

#define check_error(s) (strncmp((s), "OK",2) == 0 ? 0 : -1)

/**
 ** Open a client connection
 **/
FILE *
wmclient(int port)
{
	int fd;
	struct sockaddr_in sa;
	int reuse = 1;
	struct linger lg;
	struct protoent *pe;
	FILE *f;

	pe = getprotobyname("tcp");
	if (pe == NULL) {
		perror("getprotoent");
		return NULL;
	}
	fd = socket(PF_INET, SOCK_STREAM, pe->p_proto);
	if (fd < 0) {
		perror("socket");
		return NULL;
	}
	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse,
		sizeof(reuse)) < 0) {
		perror("setsockopt/REUSEADDR");
		return NULL;
	}
	lg.l_onoff = 0;
	lg.l_linger = 0;
	if (setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *)&lg,
		sizeof(lg)) < 0) {
		perror("setsockopt/LINGER");
		return NULL;
	}
	memset(&sa, 0, sizeof(sa));

	sa.sin_family = AF_INET;
	sa.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
	sa.sin_port = htons(port);
	
	if (connect(fd, (struct sockaddr *)&sa, sizeof(sa)) == -1) {
		fprintf(stderr, "connect: %d: %s\n",
		    port, strerror(errno));
		close(fd);
		return NULL;
	}
	f = fdopen(fd, "r+");
	if (f == NULL) {
		close(fd);
	}
	return f;
}

void
wmexit(FILE *f)
{
	fprintf(f, "EXIT\n");
	fflush(f);
	fclose(f);
}

int
wmgeom(FILE *f, const char *name, int x, int y, int width, int height)
{
	char ack[256];

	fprintf(f, "GEOM %s %d %d %d %d\n", name, x, y, width, height);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int
wmhide(FILE *f, const char *name)
{
	char ack[256];

	fprintf(f, "HIDE %s\n", name);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int
wmkill(FILE *f, const char *name)
{
	char ack[256];
	
	fprintf(f, "KILL %s\n", name);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int 
wmlow(FILE *f, const char *name)
{
	char ack[256];

	fprintf(f, "LOW %s\n", name);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}	

int 
wmnew(FILE *f, const char *cmd)
{
	char ack[256];
	
	fprintf(f, "NEW %s\n", cmd);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int 
wmshow(FILE *f, const char *name)
{
	char ack[256];

	fprintf(f, "SHOW %s\n", name);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int 
wmtodesk(FILE *f, const char *name, int n)
{
	char ack[256];
	
	fprintf(f, "TODESK %s %d\n", name, n);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int 
wmvdesk(FILE *f, int n) 
{
	char ack[256];

	fprintf(f, "VDESK %d\n", n);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int 
wmtrans(FILE *f, const char *name, double t) 
{
	char ack[256];

	fprintf(f, "TRANS %s %g\n", name, t);
	fflush(f);
	if (fgets(ack, sizeof(ack), f) == NULL) {
		return -1;
	} 
	return check_error(ack);
}

int 
wmlist(FILE *f, char *res[], int maxwin)
{
	char ack[1024];
	int i = 0;
	
	fprintf(f, "LIST\n");
	fflush(f);
	
	do {
		if (fgets(ack, sizeof(ack), f) == NULL) {
			return -1;
		}
		if (strncmp(ack, "OK", 2) == 0) {
			break;
		} else if (strncmp(ack, "LIST", 4)) {
			res[i++] = strdup(ack + 5);
		} else {
			return -1;
		}
	} while (i < maxwin);
	return i;
}

int
wmtest(FILE *f, char *cmd)
{
	char ack[256];

	if (cmd[0] == '\0') 
		return 0;

	fprintf(f, "%s\n", cmd);
	fflush(f);
	for (;;) {
		if (fgets(ack, sizeof(ack), f) == NULL) {
			return -1;
		}
		if (strncmp(ack, "OK", 2) == 0) {
			break;
		} else if (strncmp(ack, "ERROR", 5) == 0) {
			break;
		}
		printf("%s\n", ack);
	}
	return check_error(ack);
}
